@@ -4,25 +4,33 @@ use clap::{App, AppSettings, Parser, Subcommand};
 #[derive(Parser, Debug)]
 #[clap(author, version, about)]
 #[clap(global_setting(AppSettings::PropagateVersion))]
+#[clap(global_setting(AppSettings::DeriveDisplayOrder))]
+#[clap(global_setting(AppSettings::InferSubcommands))]
 #[clap(global_setting(AppSettings::UseLongFormatForHelpSubcommand))]
 #[clap(setting(AppSettings::SubcommandRequiredElseHelp))]
 pub struct Cli {
-    // Threads for decompression
+    /// threads for decompression
     #[clap(short, long, default_value_t = 8)]
     pub threads: usize,
 
+    /// logging level.
+    #[clap(short, long, parse(from_occurrences))]
+    pub verbose: usize,
+
     #[clap(subcommand)]
     pub command: Option<Commands>,
 }
 
 #[derive(Subcommand, Debug)]
 pub enum Commands {
-    /// Get percent identity stats from a sam/bam/cram or PAF
+    /// Get percent identity stats from a sam/bam/cram or PAF.
+    ///
+    /// Requires =/X operations in the CIGAR string!
     Stats {
-        // sam/bam/cram/file
+        /// sam/bam/cram/file
         #[clap(default_value = "-")]
         bam: String,
-        /// Print query coordinates first
+        /// print query coordinates first
         #[clap(short, long)]
         qbed: bool,
         /// The input is paf format
@@ -30,61 +38,22 @@ pub enum Commands {
         #[clap(short, long)]
         paf: bool,
     },
-    /// Get the frequencies of each bp at each position.
-    Nucfreq {
-        // sam/bam/cram/file
-        #[clap(default_value = "-")]
-        bam: String,
-        /// Print nucfreq info from the input region e.g "chr1:1-1000"
-        #[clap(short, long)]
-        region: Option<String>,
-        /// Print nucfreq info from regions in the bed file
-        /// output is optionally tagged using the 4th column
-        #[clap(short, long)]
-        bed: Option<String>,
-        // smaller output format
-        #[clap(short, long)]
-        small: bool,
-    },
-    /// Report the longest repeat length at every position in a fasta
-    Repeat {
-        // a fasta file
-        #[clap(default_value = "-")]
-        fasta: String,
-        /// The smallest repeat length to report
-        #[clap(short, long, default_value_t = 21)]
-        min: usize,
-    },
-    /// Extract the intervals in a genome (fasta) that are made up of SUNs
-    Suns {
-        /// fasta file with the genome
-        #[clap(short, long, default_value = "-")]
-        fasta: String,
-        /// The size of the required unique kmer
-        #[clap(short, long, default_value_t = 21)]
-        kmer_size: usize,
-        /// The maximum size SUN interval to report
-        #[clap(short, long, default_value_t = std::usize::MAX)]
-        max_size: usize,
-        /// Confirm all the SUNs (very slow) only for debugging.
-        #[clap(short, long)]
-        validate: bool,
-    },
-    /// count bases in a bed file
-    Bedlength {
-        // a bed file
+    /// Count the number of bases in a bed file.
+    #[clap(visible_aliases = &["bedlen", "bl", "bedlength"])]
+    BedLength {
+        /// a bed file
         #[clap(default_value = "-")]
         bed: String,
-        // make the output human readable (Mbp)
+        /// make the output human readable (Mbp)
         #[clap(short, long)]
         readable: bool,
     },
-    /// filter paf records in various ways
+    /// Filter PAF records in various ways.
     Filter {
         /// PAF file from minimap2 or unimap. Must have the cg tag, and n matches will be zero unless the cigar uses =X.
         #[clap(default_value = "-")]
         paf: String,
-        /// Number of aligned bases between a target and query in order to keep
+        /// minimum number of aligned bases across all alignments between a target and query in order to keep those records
         #[clap(short, long, default_value_t = 0)]
         paired_len: u64,
         /// minimum alignment length
@@ -94,36 +63,60 @@ pub enum Commands {
         #[clap(short, long, default_value_t = 0)]
         query: u64,
     },
-    ///  liftover target sequence coordinates onto query sequence using a PAF
-    Liftover {
+    /// Invert the target and query sequences in a PAF along with the CIGAR string.
+    Invert {
         /// PAF file from minimap2 or unimap. Must have the cg tag, and n matches will be zero unless the cigar uses =X.
         #[clap(default_value = "-")]
         paf: String,
-        /// Bed file of regions to liftover
+    },
+    /// Liftover target sequence coordinates onto query sequence using a PAF.
+    ///
+    /// This is a function for lifting over coordinates from a reference (--bed) to a query using a PAF file from minimap2 or unimap (note, you can use `paftools.js sam2paf` to convert SAM data to PAF format).
+    /// The returned file is a PAF file that is trimmed to the regions in the bed file. Even the cigar in the returned PAF file is trimmed so it can be used downstream! Additionally, a tag with the format `id:Z:<>` is added to the PAF where `<>` is either the 4th column of the input bed file or if not present `chr_start_end`.
+    Liftover {
+        /// PAF file from minimap2 or unimap run with -c and --eqx [i.e. the PAF file must have the cg tag and use extended CIGAR opts (=/X)].
+        #[clap(default_value = "-")]
+        paf: String,
+        /// BED file of reference regions to liftover to the query.
         #[clap(short, long)]
         bed: String,
-        /// The bed contains query coordinates to be lifted (note the query in the original PAF will become the target in the output)
+        /// Specifies that the BED file contains query coordinates to be lifted onto the reference (reverses direction of liftover).
+        ///
+        /// Note, that this will make the query in the input `PAF` the target in the output `PAF`.
         #[clap(short, long)]
         qbed: bool,
-        /// The bed contains query coordinates to be lifted (note the query in the original PAF will become the target in the output)
+        /// If multiple records overlap the same region in the <bed> return only the largest liftover. The default is to return all liftovers.
         #[clap(short, long)]
         largest: bool,
     },
-    /// orient paf records so that most of the bases are in the forward direction
+    /// Trim paf records that overlap in query sequence.
+    ///
+    /// This idea is to mimic some of the trimming that happens in PAV to improve breakpoint detection. Starts with the largest overlap and iterates until no query overlaps remain.
+    #[clap(visible_aliases = &["trim", "tp"])]
+    TrimPaf {
+        /// PAF file from minimap2 or unimap. Must have the cg tag, and n matches will be zero unless the cigar uses =X.
+        #[clap(default_value = "-")]
+        paf: String,
+    },
+    /// Orient paf records so that most of the bases are in the forward direction.
+    ///
+    /// Optionally scaffold the queriers so that there is one query per target.
     Orient {
         /// PAF file from minimap2 or unimap. Must have the cg tag, and n matches will be zero unless the cigar uses =X.
         #[clap(default_value = "-")]
         paf: String,
-        /// Make fake query names that scaffold together all the records that map to one target sequence.
-        /// The order of the scaffold will be determined by the middle position of the largest alignment.
+        /// Generate ~1 query per target that scaffolds together all the records that map to that target sequence.
+        ///
+        /// The order of the scaffold will be determined by the average target position across all the queries, and the name of the new scaffold will be
         #[clap(short, long)]
         scaffold: bool,
         /// space to add between records
         #[clap(short, long, default_value_t = 1_000_000)]
         insert: u64,
     },
-    /// break up paf on indels of a certain size
-    Breakpaf {
+    /// Break PAF records with large indels into multiple records (useful for SafFire).
+    #[clap(visible_aliases = &["breakpaf", "bp"])]
+    BreakPaf {
         /// PAF file from minimap2 or unimap. Must have the cg tag, and n matches will be zero unless the cigar uses =X.
         #[clap(default_value = "-")]
         paf: String,
@@ -131,18 +124,28 @@ pub enum Commands {
         #[clap(short, long, default_value_t = 100)]
         max_size: u32,
     },
-    /// reads in a fasta from stdin and divides into files (can compress by adding .gz)
+    /// Convert a PAF file into a SAM file. Warning, all alignments will be marked as primary!
+    #[clap(visible_aliases = &["paftosam", "p2s", "paf2sam"])]
+    PafToSam {
+        /// PAF file from minimap2 or unimap.
+        #[clap(default_value = "-")]
+        paf: String,
+    },
+    /// Reads in a fasta from stdin and divides into files (can compress by adding .gz).
+    #[clap(visible_aliases = &["fastasplit", "fasplit"])]
     FastaSplit {
         /// list of fasta files
         fasta: Vec<String>,
     },
-    /// reads in a fastq from stdin and divides into files (can compress by adding .gz)
+    /// Reads in a fastq from stdin and divides into files (can compress by adding .gz).
+    #[clap(visible_aliases = &["fastqsplit", "fqsplit"])]
     FastqSplit {
         /// list of fastq files
         fastq: Vec<String>,
     },
     /// Mimic bedtools getfasta but allow for bgzip in both bed and fasta inputs.
-    Getfasta {
+    #[clap(visible_aliases = &["getfasta", "gf"])]
+    GetFasta {
         /// fasta file to extract sequences from
         #[clap(short, long, default_value = "-")]
         fasta: String,
@@ -156,6 +159,46 @@ pub enum Commands {
         #[clap(short, long)]
         name: bool,
     },
+    /// Get the frequencies of each bp at each position.
+    Nucfreq {
+        // sam/bam/cram/file
+        #[clap(default_value = "-")]
+        bam: String,
+        /// print nucfreq info from the input region e.g "chr1:1-1000"
+        #[clap(short, long)]
+        region: Option<String>,
+        /// print nucfreq info from regions in the bed file
+        /// output is optionally tagged using the 4th column
+        #[clap(short, long)]
+        bed: Option<String>,
+        // smaller output format
+        #[clap(short, long)]
+        small: bool,
+    },
+    /// Report the longest exact repeat length at every position in a fasta.
+    Repeat {
+        // a fasta file
+        #[clap(default_value = "-")]
+        fasta: String,
+        /// The smallest repeat length to report
+        #[clap(short, long, default_value_t = 21)]
+        min: usize,
+    },
+    /// Extract the intervals in a genome (fasta) that are made up of SUNs.
+    Suns {
+        /// fasta file with the genome
+        #[clap(short, long, default_value = "-")]
+        fasta: String,
+        /// the size of the required unique kmer
+        #[clap(short, long, default_value_t = 21)]
+        kmer_size: usize,
+        /// the maximum size SUN interval to report
+        #[clap(short, long, default_value_t = std::usize::MAX)]
+        max_size: usize,
+        /// confirm all the SUNs (very slow) only for debugging.
+        #[clap(short, long)]
+        validate: bool,
+    },
 }
 
 pub fn make_cli_parse() -> Cli {
