@@ -1,14 +1,19 @@
 use bio::io::fasta;
 use bio::io::fastq;
+use colored::Colorize;
 use itertools::Itertools;
 use rayon::prelude::*;
 use rust_htslib::bam;
 use rust_htslib::bam::Read;
 use rustybam::cli::Commands;
+use rustybam::paf::paf_swap_query_and_target;
 use rustybam::*;
 use std::io;
 use std::time::Instant;
 
+use env_logger::{Builder, Target};
+use log::LevelFilter;
+
 fn main() {
     parse_cli();
 }
@@ -19,6 +24,21 @@ pub fn parse_cli() {
     let matches = cli::make_cli_app().get_matches();
     let subcommand = matches.subcommand_name().unwrap();
 
+    // set the logging level
+    let min_log_level = match matches.occurrences_of("verbose") {
+        0 => LevelFilter::Warn,
+        1 => LevelFilter::Info,
+        2 => LevelFilter::Debug,
+        _ => LevelFilter::Trace,
+    };
+    Builder::new()
+        .target(Target::Stderr)
+        .filter(None, min_log_level)
+        .init();
+
+    log::debug!("DEBUG logging enabled");
+    log::trace!("TRACE logging enabled");
+
     // set up number of threads to use globally
     rayon::ThreadPoolBuilder::new()
         .num_threads(args.threads)
@@ -142,7 +162,7 @@ pub fn parse_cli() {
         //
         // Run Bedlength
         //
-        Some(Commands::Bedlength { bed, readable }) => {
+        Some(Commands::BedLength { bed, readable }) => {
             let rgns = bed::parse_bed(bed);
             let count: u64 = rgns.into_iter().map(|rgn| rgn.en - rgn.st).sum();
             if *readable {
@@ -152,6 +172,16 @@ pub fn parse_cli() {
             }
         }
         //
+        // Run Invert
+        //
+        Some(Commands::Invert { paf }) => {
+            let paf = paf::Paf::from_file(paf);
+            for rec in &paf.records {
+                let inv_rec = paf_swap_query_and_target(rec);
+                println!("{}", inv_rec);
+            }
+        }
+        //
         // Run Liftover
         //
         Some(Commands::Liftover {
@@ -183,6 +213,16 @@ pub fn parse_cli() {
             }
         }
         //
+        // Run TrimPaf
+        //
+        Some(Commands::TrimPaf { paf }) => {
+            let mut paf = paf::Paf::from_file(paf);
+            paf.overlapping_paf_recs();
+            for rec in &paf.records {
+                println!("{}", rec);
+            }
+        }
+        //
         // Run Filter
         //
         Some(Commands::Filter {
@@ -192,11 +232,11 @@ pub fn parse_cli() {
             query,
         }) => {
             let mut paf = paf::Paf::from_file(paf);
-            eprintln!("{} PAF records BEFORE filtering.", paf.records.len());
+            log::info!("{} PAF records BEFORE filtering.", paf.records.len());
             paf.filter_query_len(*query);
             paf.filter_aln_len(*aln);
             paf.filter_aln_pairs(*paired_len);
-            eprintln!("{} PAF records AFTER filtering.", paf.records.len());
+            log::info!("{} PAF records AFTER filtering.", paf.records.len());
             for rec in paf.records {
                 println!("{}", rec);
             }
@@ -222,7 +262,7 @@ pub fn parse_cli() {
         //
         // Run Breakpaf
         //
-        Some(Commands::Breakpaf { paf, max_size }) => {
+        Some(Commands::BreakPaf { paf, max_size }) => {
             // read in the file
             let paf = paf::Paf::from_file(paf);
             for mut paf in paf.records {
@@ -234,6 +274,16 @@ pub fn parse_cli() {
             }
         }
         //
+        // Run PafToSam
+        //
+        Some(Commands::PafToSam { paf }) => {
+            let paf = paf::Paf::from_file(paf);
+            println!("{}", paf.sam_header());
+            for rec in paf.records {
+                println!("{}", rec.to_sam_string());
+            }
+        }
+        //
         // Run Fasta-split
         //
         Some(Commands::FastaSplit { fasta }) => {
@@ -246,9 +296,9 @@ pub fn parse_cli() {
             run_split_fastq(fastq);
         }
         //
-        // Run Getfasta
+        // Run GetFasta
         //
-        Some(Commands::Getfasta {
+        Some(Commands::GetFasta {
             fasta,
             bed,
             strand,
@@ -263,9 +313,10 @@ pub fn parse_cli() {
     };
 
     let duration = pg_start.elapsed();
-    eprintln!(
-        "[SUCCESS] Time elapsed during rustybam-{}: {:.3?}",
-        subcommand, duration
+    log::info!(
+        "rustybam-{} done! Time elapsed: {:.2?}",
+        subcommand.bright_green().bold(),
+        duration
     );
 }
 
