@@ -1,7 +1,10 @@
+use crate::trim_overlap::trim_overlapping_pafs;
+
 use super::bed;
 use core::{fmt, panic};
 use itertools::Itertools;
 use lazy_static::lazy_static;
+use natord;
 use regex::Regex;
 use rust_htslib::bam::record::Cigar::*;
 use rust_htslib::bam::record::CigarString;
@@ -27,16 +30,16 @@ pub enum Error {
 type PafResult<T> = Result<T, crate::paf::Error>;
 
 #[derive(Debug)]
-pub struct Paf<'a> {
+pub struct Paf {
     pub records: Vec<PafRecord>,
-    pub records_by_contig: HashMap<String, Vec<&'a PafRecord>>,
+    //pub records_by_contig: HashMap<String, Vec<&'a PafRecord>>,
 }
 
-impl<'a> Paf<'a> {
-    fn new() -> Paf<'a> {
+impl Paf {
+    fn new() -> Paf {
         Paf {
             records: Vec::new(),
-            records_by_contig: HashMap::new(),
+            //records_by_contig: HashMap::new(),
         }
     }
     /// read in the paf from a file pass "-" for stdin
@@ -49,29 +52,41 @@ impl<'a> Paf<'a> {
     /// assert_eq!(paf.records.len(), 249);
     ///
     /// ```
-    pub fn from_file(file_name: &str) -> Paf<'a> {
+    pub fn from_file(file_name: &str) -> Paf {
         // open the paf file
         let paf_file: Box<dyn io::Read> = match file_name {
             "-" => Box::new(io::stdin()),
             _ => Box::new(fs::File::open(file_name).expect("Unable to open paf file")),
         };
         let mut paf = Paf::new();
-        let mut records = Vec::new();
+        //let mut records = Vec::new();
         // read the paf recs into a vector
         for (index, line) in io::BufReader::new(paf_file).lines().enumerate() {
             match PafRecord::new(&line.unwrap()) {
                 Ok(rec) => {
                     //eprint!("\rReading PAF entry # {}", index);
-                    records.push(rec)
+                    paf.records.push(rec);
+                    //let q_name = rec.q_name.clone();
+                    //let rec = paf.records.last().expect("must have inserted at least one");
+                    //(*paf.records_by_contig.entry(q_name).or_insert(Vec::new())).push(x);
                 }
                 Err(_) => eprintln!("\nUnable to parse PAF record. Skipping line {}", index + 1),
             }
         }
-        eprintln!();
-        paf.records = records;
         paf
     }
 
+    pub fn query_name_map(&mut self) -> HashMap<String, Vec<&PafRecord>> {
+        let mut records_by_contig = HashMap::new();
+        for rec in &self.records {
+            (*records_by_contig
+                .entry(rec.q_name.clone())
+                .or_insert_with(Vec::new))
+            .push(rec);
+        }
+        records_by_contig
+    }
+
     pub fn filter_aln_pairs(&mut self, paired_len: u64) {
         let mut dict = HashMap::new();
         for rec in self.records.iter_mut() {
@@ -140,7 +155,7 @@ impl<'a> Paf<'a> {
         }
     }
 
-    // scaffold oriented contigs into one fake super contig
+    /// scaffold oriented contigs into one fake super contig
     pub fn scaffold(&mut self, spacer_size: u64) {
         // sort the records by their target name and order
         self.records.sort_by(|a, b| {
@@ -189,6 +204,96 @@ impl<'a> Paf<'a> {
             }
         }
     }
+
+    /// Identify overlapping pairs in Paf set
+    pub fn overlapping_paf_recs(&mut self) {
+        let mut overlap_pairs = Vec::new();
+        self.records.sort_by_key(|rec| rec.q_name.clone());
+
+        for i in 0..(self.records.len() - 1) {
+            let rec1 = &self.records[i];
+            let rgn1 = rec1.get_query_as_region();
+            let mut j = i + 1;
+            while j < self.records.len() && rec1.q_name == self.records[j].q_name {
+                let rec2 = &self.records[j];
+                let rgn2 = rec2.get_query_as_region();
+                // count overlap
+                let overlap = bed::get_overlap(&rgn1, &rgn2);
+                // check if rec2 is contained
+                if overlap < 1 {
+                    j += 1;
+                    continue;
+                } else if overlap == (rec2.q_en - rec2.q_st) {
+                    //rec2.contained = true;
+                    log::trace!("{}\n^is contained in another alignment", rec1);
+                } else if overlap == (rec1.q_en - rec1.q_st) {
+                    //rec1.contained = true;
+                    log::trace!("{}\n^is contained in another alignment", rec1);
+                } else {
+                    // put recs in left, right order
+                    if rec1.q_st <= rec2.q_st {
+                        overlap_pairs.push((overlap, i, j));
+                    } else {
+                        overlap_pairs.push((overlap, j, i));
+                    }
+                }
+                // go to next
+                j += 1;
+            }
+        }
+        overlap_pairs.sort_by_key(|rec| std::u64::MAX - rec.0);
+        if overlap_pairs.len() > 1 {
+            let mut left = self.records[overlap_pairs[0].1].clone();
+            let mut right = self.records[overlap_pairs[0].2].clone();
+            left.aligned_pairs();
+            right.aligned_pairs();
+            trim_overlapping_pafs(&mut left, &mut right);
+            log::trace!("{}", left);
+            log::trace!("{}", right);
+            self.records[overlap_pairs[0].1] = left;
+            self.records[overlap_pairs[0].2] = right;
+            // recursively call for next overlap
+            self.overlapping_paf_recs();
+        }
+    }
+
+    /// Make a SAM header from a Paf
+    /// # Example
+    /// ```
+    /// use rustybam::paf;
+    /// use std::fs::File;
+    /// use std::io::*;
+    /// let mut paf = paf::Paf::from_file(".test/asm_small.paf");
+    /// let header = paf.sam_header();
+    /// assert_eq!(header[0..3], "@HD".to_string());
+    /// assert_eq!(header.split("\n").count(), 5);
+    /// ```
+    pub fn sam_header(&self) -> String {
+        /*
+        @HD	VN:1.6	SO:coordinate
+        @SQ	SN:chr1	LN:248387497
+        ...
+        @SQ	SN:chrM	LN:16569
+        @SQ	SN:chrY	LN:57227415
+        @PG	ID:unimap	PN:unimap	VN:0.1-r41	CL:
+        */
+        let mut header = "@HD\tVN:1.6\n".to_string();
+
+        // sort names naturally
+        let mut names: Vec<(String, u64)> = self
+            .records
+            .iter()
+            .map(|rec| (rec.t_name.clone(), rec.t_len))
+            .unique()
+            .collect();
+
+        names.sort_by(|a, b| natord::compare(&a.0, &b.0));
+        for (name, length) in names {
+            header.push_str(&format!("@SQ\tSN:{}\tLN:{}\n", name, length));
+        }
+        header.push_str("@PG\tID:rustybam\tPN:rustybam");
+        header
+    }
 }
 
 #[derive(Debug, Clone)]
@@ -212,6 +317,7 @@ pub struct PafRecord {
     pub long_cigar: CigarString,
     pub id: String,
     pub order: u64,
+    pub contained: bool,
 }
 
 impl PafRecord {
@@ -268,6 +374,7 @@ impl PafRecord {
             long_cigar: CigarString(Vec::new()),
             id: "".to_string(),
             order: 0,
+            contained: false,
         };
         Ok(rec)
     }
@@ -293,6 +400,37 @@ impl PafRecord {
             long_cigar: CigarString(Vec::new()),
             id: self.id.clone(),
             order: self.order,
+            contained: self.contained,
+        }
+    }
+
+    /// This function returns a region type from the paf query
+    pub fn get_query_as_region(&self) -> bed::Region {
+        bed::Region {
+            name: self.q_name.clone(),
+            st: self.q_st,
+            en: self.q_en,
+            ..Default::default()
+        }
+    }
+
+    /// This function returns a region type from the paf target
+    /// Example:
+    /// ```
+    /// use rustybam::bed::*;
+    /// use rustybam::paf::*;
+    /// let paf = PafRecord::new("Q 10 0 10 + T 20 12 20 3 9 60 cg:Z:7=1X2=").unwrap().get_target_as_region();
+    /// let rgn = Region {name: "T".to_string(), st: 12, en: 20, ..Default::default()};
+    /// assert_eq!((rgn.name, rgn.st, rgn.en),
+    ///             (paf.name, paf.st, paf.en)
+    /// );
+    /// ```
+    pub fn get_target_as_region(&self) -> bed::Region {
+        bed::Region {
+            name: self.t_name.clone(),
+            st: self.t_st,
+            en: self.t_en,
+            ..Default::default()
         }
     }
 
@@ -333,7 +471,7 @@ impl PafRecord {
         self.long_cigar = CigarString(long_cigar);
     }
 
-    // Does a binary search on the alignment to find its index in the alignment
+    /// Does a binary search on the alignment to find its index in the alignment
     pub fn tpos_to_idx(&self, tpos: u64, right: bool) -> Result<usize, usize> {
         let mut idx = self.tpos_aln.binary_search(&tpos)?;
         if right {
@@ -348,7 +486,28 @@ impl PafRecord {
         Ok(idx)
     }
 
-    // subset cigar on coordiantes
+    /// Does a binary search on the query sequence to find its index in the alignment
+    pub fn qpos_to_idx(&self, qpos: u64) -> Result<usize, usize> {
+        let idx = if self.strand == '-' {
+            // can be reverse sorted if the strand is "-" so need to mod bin search
+            self.qpos_aln
+                .binary_search_by(|probe| probe.cmp(&qpos).reverse())?
+        } else {
+            self.qpos_aln.binary_search(&qpos)?
+        };
+        /*if right {
+            while idx + 1 < self.qpos_aln.len() && self.qpos_aln[idx + 1] == qpos {
+                idx += 1;
+            }
+        } else {
+            while idx > 0 && self.qpos_aln[idx - 1] == qpos {
+                idx -= 1;
+            }
+        }*/
+        Ok(idx)
+    }
+
+    /// subset cigar on coordinates
     pub fn subset_cigar(&self, start_idx: usize, end_idx: usize) -> CigarString {
         // update the cigar string
         let mut new_cigar = vec![];
@@ -386,7 +545,7 @@ impl PafRecord {
     }
 
     /// Return a tuple with the n of bases in the query and
-    /// target infered from the cigar string
+    /// target inferred from the cigar string
     pub fn infer_n_bases(&mut self) -> (u64, u64, u64, u64) {
         let mut t_bases = 0;
         let mut q_bases = 0;
@@ -487,11 +646,11 @@ impl PafRecord {
         self.cigar = CigarString(self.cigar.0[remove_st_opts..].to_vec());
         self.cigar.0.truncate(self.cigar.len() - remove_en_opts);
 
-        // update the target coordiantes
+        // update the target coordinates
         self.t_st += remove_st_t as u64;
         self.t_en -= remove_en_t as u64;
 
-        // update the query coordiantes if rc
+        // update the query coordinates if rc
         if self.strand == '-' {
             std::mem::swap(&mut remove_st_q, &mut remove_en_q);
         }
@@ -510,6 +669,42 @@ impl PafRecord {
         }
     }
 
+    pub fn truncate_record_by_query(&mut self, new_q_st: u64, new_q_en: u64) {
+        // checks
+        assert!(new_q_st >= self.q_st, "New start is less than old start.");
+        assert!(new_q_en <= self.q_en, "New end is greater than old end.");
+
+        // get alignment positions
+        let mut aln_st = self.qpos_to_idx(new_q_st).unwrap();
+        let mut aln_en = self.qpos_to_idx(new_q_en - 1).unwrap();
+        if aln_st > aln_en {
+            std::mem::swap(&mut aln_st, &mut aln_en);
+        }
+        let new_t_st = self.tpos_aln[aln_st];
+        let new_t_en = self.tpos_aln[aln_en] + 1; // ends are not inclusive
+
+        // update the cigar string
+        self.long_cigar = self.subset_cigar(aln_st, aln_en);
+        self.cigar = PafRecord::collapse_long_cigar(&self.long_cigar);
+
+        // update the target coordinates
+        self.t_st = new_t_st;
+        self.t_en = new_t_en;
+
+        // fix the query positions that need to be
+        self.q_st = new_q_st;
+        self.q_en = new_q_en;
+
+        // update alignment positions
+        self.aligned_pairs();
+
+        // should not happen but just in case
+        self.remove_trailing_indels();
+
+        // check integrity and update aln_len and nmatch
+        self.check_integrity().unwrap();
+    }
+
     pub fn check_integrity(&mut self) -> PafResult<()> {
         let (t_bases, q_bases, nmatch, aln_len) = self.infer_n_bases();
         if self.t_en - self.t_st != t_bases {
@@ -543,6 +738,30 @@ impl PafRecord {
 
         Ok(())
     }
+
+    /// Print the paf record as a SAM record
+    /// Example:
+    /// ```
+    /// use rustybam::bed::*;
+    /// use rustybam::paf::*;
+    /// let paf = PafRecord::new("Q 10 0 10 + T 20 12 20 3 9 60 cg:Z:7=1X2=").unwrap();
+    /// let sam = paf.to_sam_string();
+    /// ```
+    pub fn to_sam_string(&self) -> String {
+        /*
+        m64062_190807_194840/133628256/ccs	0	chr1	1	60	396=	*	0	0   *   *
+        */
+        let flag = if self.strand == '-' { 16 } else { 0 };
+        format!(
+            "{}\t{}\t{}\t{}\t{}\t{}\t*\t0\t0\t*\t*",
+            self.q_name,
+            flag,
+            self.t_name,
+            self.q_st + 1,
+            self.mapq,
+            self.cigar
+        )
+    }
 }
 
 impl fmt::Display for PafRecord {
@@ -718,86 +937,6 @@ pub fn paf_swap_query_and_target(paf: &PafRecord) -> PafRecord {
     flipped
 }
 
-pub fn old_trim_paf_rec_to_rgn(rgn: &bed::Region, paf: &PafRecord) -> PafRecord {
-    // initalize a trimmed paf record
-    let mut trimmed_paf = (*paf).clone();
-    // check if we can return right away
-    if paf.t_st >= rgn.st && paf.t_en <= rgn.en {
-        return trimmed_paf;
-    }
-    // count until we get to the correct regions
-    let mut t_pos = paf.t_st;
-    let mut q_offset = 0;
-    let mut q_offset_st = 0;
-    let mut q_offset_en = 0;
-    let mut new_cigar = Vec::new();
-    trimmed_paf.nmatch = 0;
-    trimmed_paf.aln_len = 0;
-    for opt in paf.cigar.into_iter() {
-        let moves_t = consumes_reference(opt);
-        let moves_q = consumes_query(opt);
-        let opt_len = opt.len() as u64;
-        let mut new_opt_len = 0;
-        let mut done = false;
-        // incrment the ref and or query
-        for _i in 0..opt_len {
-            if moves_t {
-                t_pos += 1;
-            }
-            if moves_q {
-                q_offset += 1;
-            }
-            // add to the new cigar opt length
-            if t_pos > rgn.st && t_pos <= rgn.en {
-                new_opt_len += 1;
-                trimmed_paf.aln_len += 1;
-            }
-            // record the starting positions
-            if moves_t && t_pos == rgn.st {
-                trimmed_paf.t_st = rgn.st;
-                q_offset_st = q_offset;
-            }
-            // record the ending positions
-            if moves_t && (t_pos == rgn.en || t_pos == paf.t_en) {
-                trimmed_paf.t_en = t_pos;
-                q_offset_en = q_offset;
-                done = true;
-                break;
-            }
-        }
-        // add to the new cigar string.
-        if new_opt_len != 0 {
-            new_cigar.push(match opt {
-                Match(_) => Match(new_opt_len),
-                Ins(_) => Ins(new_opt_len),
-                Del(_) => Del(new_opt_len),
-                RefSkip(_) => RefSkip(new_opt_len),
-                HardClip(_) => HardClip(new_opt_len),
-                SoftClip(_) => SoftClip(new_opt_len),
-                Pad(_) => Pad(new_opt_len),
-                Equal(_) => {
-                    trimmed_paf.nmatch += new_opt_len as u64;
-                    Equal(new_opt_len)
-                }
-                Diff(_) => Diff(new_opt_len),
-            })
-        }
-        if done {
-            break;
-        }
-    }
-    // fix strand if needed
-    if paf.strand == '+' {
-        trimmed_paf.q_st = paf.q_st + q_offset_st;
-        trimmed_paf.q_en = paf.q_st + q_offset_en;
-    } else {
-        trimmed_paf.q_st = paf.q_en - q_offset_en + 1;
-        trimmed_paf.q_en = paf.q_en - q_offset_st;
-    }
-    trimmed_paf.cigar = CigarString(new_cigar); // update to trimmed cigar.
-    trimmed_paf
-}
-
 pub fn make_fake_paf_rec() -> PafRecord {
     let mut rtn = PafRecord::new("Q 10 2 10 - T 20 12 20 3 9 60 cg:Z:4M1I1D3=").unwrap();
     rtn.aligned_pairs();
